@@ -27,7 +27,6 @@ def load_and_init_from_playback_file(filename, mock_urlopen):
     mock_url_open_object = mock.Mock()
     mock_url_open_object.read.side_effect = [
         pb['watch_html'].encode('utf-8'),
-        pb['vid_info_raw'].encode('utf-8'),
         pb['js'].encode('utf-8')
     ]
     mock_urlopen.return_value = mock_url_open_object
@@ -39,10 +38,9 @@ def load_and_init_from_playback_file(filename, mock_urlopen):
     #  deferred
     v = YouTube(pb["url"])
     v.watch_html
-    v.vid_info_raw
+    v._vid_info = pb['vid_info']
     v.js
     v.fmt_streams
-    v.player_response
     return v
 
 
@@ -81,13 +79,6 @@ def missing_recording():
     return load_playback_file(filename)
 
 
-@pytest.fixture
-def region_blocked():
-    """Youtube instance initialized with video id hZpzr8TbF08."""
-    filename = "yt-video-hZpzr8TbF08-html.json.gz"
-    return load_playback_file(filename)
-
-
 @pytest.fixture
 def playlist_html():
     """Youtube playlist HTML loaded on 2020-01-25 from
